<?php
namespace ystp;

class Actions {
	public $customPostTypeObj;

	public function __construct() {
		$this->init();
	}

	public function init() {
		add_action('init', array($this, 'postTypeInit'));
		add_action('admin_menu', array($this, 'addSubMenu'));
		add_action('add_meta_boxes', array($this, 'addMetaboxes'), 10, 2);
		add_action('save_post_'.YSTP_POST_TYPE, array($this, 'save'), 10, 3);
		add_action('manage_'.YSTP_POST_TYPE.'_posts_custom_column' , array($this, 'tableColumnValues'), 10, 2);
		add_action('wp_enqueue_scripts', array($this, 'wpEnqueueScript'));
        add_action('admin_head', array($this, 'adminHead'));
	}

    public function adminHead() {
        echo "<script>jQuery(document).ready(function() {jQuery('a[href*=\"page=ystp-support-page\"]').css({color: 'yellow'});jQuery('a[href*=\"page=ystp-support-page\"]').bind('click', function(e) {e.preventDefault(); window.open('https://wordpress.org/support/plugin/scroll-to-top-builder/')}) });</script>";
    }
	
	public function tableColumnValues($column, $postId) {
		$scroll = Scroll::find($postId);
		if($column == 'ystpType') {
			if(is_object($scroll)) {
				echo $scroll->getDisplayName();
			}
			else {
				_e('Unknown type', YSTP_TEXT_DOMAIN);
			}
		}
		if($column == 'onof') {
			$checked = '';
			$isActive = Scroll::isActivePost($postId);
			
			if ($isActive) {
				$checked = 'checked';
			}
			
			?>
			<label class="ystp-switch">
				<input type="checkbox" data-id="<?php echo esc_attr($postId); ?>" name="ystp-show-status" class="ystp-accordion-checkbox ystp-show-status" <?php echo $checked; ?> >
				<span class="ystp-slider ystp-round"></span>
			</label>
			<?php
		}
	}
	
	public function wpEnqueueScript() {
		$include = new IncludeToPage();
		$include->includeData();
	}
	
	public function hideMetaboxes($hidden, $screen) {
		if (('post' == $screen->base) && (YSTP_POST_TYPE == $screen->id)) {
			$hidden[] ='hiddenMetaboxes';//for custom meta box, enter the id used in the add_meta_box() function.
		}
		
		return $hidden;
	}
	
	public function save($postId, $post, $update) {
		if(!$update) {
			return false;
		}
		$safePost = filter_input_array(INPUT_POST);
		$postData = Scroll::parseScrollDataFromData($safePost);

		$postData = apply_filters('ystpSavedData', $postData);
		if(empty($postData)) {
			return false;
		}
		$postData['ystp-post-id'] = $postId;

		if (!empty($postData['ystp-type'])) {
			$type = $postData['ystp-type'];
			$typePath = Scroll::getTypePathFormScrollType($type);
			$className = Scroll::getClassNameScrollType($type);
			
			require_once($typePath.$className.'.php');
			$className = __NAMESPACE__.'\\'.$className;
			
			$className::create($postData);
		}
		return true;
	}
	
	public function defaultMainMetaboxes() {
		$metaboxes = array();
		
		$metaboxes['hiddenMetaboxes'] = array('title' => 'hidden options', 'position' => 'normal', 'prioritet' => 'high');
		$metaboxes['generalMetaboxes'] = array('title' => 'General options', 'position' => 'normal', 'prioritet' => 'high');
		$metaboxes['displaySettings'] = array('title' => 'Display settingss', 'position' => 'normal', 'prioritet' => 'high');
		$metaboxes['conditionsSection'] = array('title' => 'Conditions', 'position' => 'normal', 'prioritet' => 'high');
		
		return $metaboxes;
	}
	
	public function getMetaBoxes() {
		$metaboxes = $this->defaultMainMetaboxes();
		
		return apply_filters('ystpGeneralMetaboxes', $metaboxes);
	}
	
	public function addMetaboxes() {
		$metaboxes = $this->getMetaBoxes();
		
		foreach ($metaboxes as $key => $metabox) {
			$obj = $this;
			if(isset($metabox['currentObj'])) {
				$obj = $metabox['currentObj'];
			}
			add_meta_box($key, __($metabox['title'], YSTP_TEXT_DOMAIN), array($obj, $key), YSTP_POST_TYPE, $metabox['position'], $metabox['prioritet']);
		}
	}
	
	public function hiddenMetaboxes() {
		$typeObj = $this->customPostTypeObj->getTypeObj();
		require_once YSTP_VIEWS_PATH.'hiddenOptions.php';
	}
	
	public function generalMetaboxes() {
		$typeObj = $this->customPostTypeObj->getTypeObj();
		require_once YSTP_VIEWS_PATH.'generalOptions.php';
	}

	public function displaySettings() {
		$typeObj = $this->customPostTypeObj->getTypeObj();
		require_once YSTP_VIEWS_PATH.'displaySettings.php';
	}

	public function conditionsSection() {
		if(YSTP_PKG_VERSION == YSTP_FREE_VERSION) {
			return require_once YSTP_VIEWS_PATH.'conditionsFreeSection.php';
		}
		$typeObj = $this->customPostTypeObj->getTypeObj();
		require_once YSTP_VIEWS_PATH.'conditionsSection.php';
	}

	public function postTypeInit(){
		$this->customPostTypeObj = new RegisterPostType();
		add_filter('default_hidden_meta_boxes', array($this, 'hideMetaboxes'),10,2);
	}
	
	public function addSubMenu() {
		$this->customPostTypeObj->addSubMenu();
	}
}
